from sqlobject import *
from extracols import DictPickleCol

import emails

from quixote import get_request

import storage

from form import *
from misc import simplify

from formdata import FormData
from anonylink import AnonymityLink

widget_classes = {
    'string': StringWidget,
    'email': EmailWidget,
    'text': TextWidget,
    'bool': CheckboxWidget,
    'item': SingleSelectWidget,
    'file': FileWidget,
}

widget_extra_attributes = {
    'text': ('cols', 'rows'),
    'string': ('size',),
}


class ConsultationField(SQLObject):
    label = StringCol(length = 300)
    type = StringCol(length = 20)
    required = BoolCol(default = True)
    extra = DictPickleCol(default = None)
    consultation = ForeignKey('Consultation')
    position = IntCol(default = None)



class Consultation(SQLObject):
    key = 'id'
    names = 'consultations'

    name = StringCol(length = 200)
    url_name = StringCol(default = None, length = 200, alternateID = True)
    fields = MultipleJoin('ConsultationField', joinMethodName = 'fields',
            joinColumn = 'consultation_id', orderBy = ConsultationField.q.position)
    receiver = ForeignKey('Role', default = None)
    roles = RelatedJoin('Role')
    category = ForeignKey('Category', default = None)

    def _set_name(self, value):
        self._SO_set_name(value)
        if not self.url_name or FormData.select(FormData.q.consultationID == self.id).count() == 0:
            self.url_name = simplify(value)

    def create_form(self, page_no, answers = {}):
        form = Form(enctype = "multipart/form-data", use_tokens = False)
        current_page = 0
        for field in self.fields:
            if field.type == 'newpage':
                current_page += 1
                if current_page > page_no:
                    break
                continue
            if current_page != page_no:
                continue
            if field.type == 'title':
                form.widgets.append(HtmlWidget(htmltext("<h3>%s</h3>" % field.label)))
                continue
            if field.type == 'subtitle':
                form.widgets.append(HtmlWidget(htmltext("<h4>%s</h4>" % field.label)))
                continue
            if field.type == 'comment':
                form.widgets.append(HtmlWidget(htmltext("<p>%s</p>" % field.label)))
                continue

            widget_class = widget_classes[field.type]

            kwargs = {'required': field.required}
            if field.type == 'item':
                kwargs['options'] = field.extra.get('items', ['---'])
                if field.extra.get('show-as-radio'):
                    widget_class = RadiobuttonsWidget
                    if len(kwargs['options']) > 2:
                        kwargs['delim'] = htmltext('<br />')
            kwargs['value'] = answers.get(field.id)
            if widget_extra_attributes.has_key(field.type):
                for k in widget_extra_attributes[field.type]:
                    v = field.extra.get(k, None)
                    if v is not None:
                        kwargs[k] = v
            form.add(widget_class, "f%d" % field.id, title = field.label, **kwargs)

        if page_no != 0:
            form.add_submit("previous", _("Previous"))
        form.add_submit("submit", _("Next"))

        return form

    def get_data(self, form):
        d = {}
        for field in self.fields:
            widget = form.get_widget('f%s' % field.id)
            if widget:
                d[field.id] = widget.parse()
        return d

    def get_filled(self, session):
        completed = FormData.select(AND(
                    FormData.q.consultationID == self.id,
                    FormData.q.userID == session.user))
        if completed.count():
            return completed[0]

        completed = AnonymityLink.select(OR(
                        AnonymityLink.q.name_identifier == session.name_identifier,
                        AND(AnonymityLink.q.key != None,
                            AnonymityLink.q.key == session.get_anonymous_key(False))))
        for c in completed:
            if c.formdata.consultationID == self.id:
                return c.formdata

        raise SQLObjectNotFound

    def migrateOldData(cls):
        import storage
        from roles import Role, get_logged_users_role
        from categories import Category

        init_method = cls.__init__
        cls.__init__ = lambda x: x
        consultations = storage.get_storage().values('consultations')
        items = [x.__dict__ for x in consultations]
        cls.__init__ = init_method

        items.sort(lambda x,y: cmp(x['id'], y['id']))
        logged_users = get_logged_users_role()
        for item in items:
            consultation = Consultation(name = item['name'])
            consultation.url_name = item['id']
            consultation.receiver = Role.get(item['receiver'])
            if item.get('category'):
                consultation.category = Category.get(item['category'])
            for role in item.get('roles', []):
                if role == 'logged-users':
                    consultation.addRole(logged_users)
                    continue
                consultation.addRole(Role.get(role))
            for i, field in enumerate(item['questions']):
                fieldObject = ConsultationField(label = field['question'], type = field['answer'],
                        consultation = consultation)
                fieldObject.required = field.get('required', False)
                fieldObject.position = i
                d = field.copy()
                del d['question']
                del d['answer']
                del d['required']
                if d:
                    fieldObject.extra = d
            
            # convert submitted forms
            name = 'consultation-%s' % item['id']
            newname = 'consultationtmp-%s' % consultation.id
            storage.get_storage().rename_table(name, newname)

    migrateOldData = classmethod(migrateOldData)
