from sqlobject import *
from extracols import DictPickleCol
from quixote import get_request, get_session

import emails

from form import *
from misc import simplify

from formdata import FormData

widget_classes = {
    'string': StringWidget,
    'email': EmailWidget,
    'text': TextWidget,
    'bool': CheckboxWidget,
    'item': SingleSelectWidget,
    'file': FakeFileWidget,
}

widget_extra_attributes = {
    'text': ('cols', 'rows'),
    'string': ('size',),
}

status_labels = {
    None: N_('Unknown'),
    'new': N_('New'),
    'rejected': N_('Rejected'),
    'accepted': N_('Accepted'),
    'finished': N_('Finished')
}

class FormField(SQLObject):
    label = StringCol(length = 300)
    type = StringCol(length = 20)
    required = BoolCol(default = True)
    extra = DictPickleCol(default = None)
    formdef = ForeignKey('FormDef')
    position = IntCol(default = None)


class FormDef(SQLObject):
    key = 'id'
    names = 'formdefs'

    name = StringCol(length = 200)
    url_name = StringCol(default = None, length = 200, alternateID = True)
    fields = MultipleJoin('FormField', joinMethodName = 'fields', joinColumn = 'formdef_id',
            orderBy = FormField.q.position)
    receiver = ForeignKey('Role', default = None)
    category = ForeignKey('Category', default = None)
    roles = RelatedJoin('Role')
    discussion = BoolCol(default = False)
    confirmation = BoolCol(default = True)
    public = BoolCol(default = False)
    detailed_emails = BoolCol(default = False)

    def _set_name(self, value):
        self._SO_set_name(value)
        if not self.url_name or FormData.select(FormData.q.formdefID == self.id).count() == 0:
            self.url_name = simplify(value)

    def create_form(self):  # XXX: merge create_form and create_view_form
        form = Form(enctype = "multipart/form-data", use_tokens = not self.confirmation)
        for field in self.fields:
            if field.type == 'title':
                form.widgets.append(HtmlWidget(htmltext('<h3>%s</h3>' % field.label)))
                continue
            if field.type == 'subtitle':
                form.widgets.append(HtmlWidget(htmltext('<h4>%s</h4>' % field.label)))
                continue
            if field.type == 'comment':
                form.widgets.append(HtmlWidget(htmltext('<p>%s</p>' % field.label)))
                continue
            
            widget_class = widget_classes[field.type]

            kwargs = {'required': field.required}
            if field.type == 'item':
                kwargs['options'] = field.extra.get('items', ['---'])
                if field.extra.get('show-as-radio'):
                    widget_class = RadiobuttonsWidget
            if widget_extra_attributes.has_key(field.type):
                for k in widget_extra_attributes[field.type]:
                    v = field.extra.get(k)
                    if v:
                        kwargs[k] = v
            form.add(widget_class, 'f%d' % field.id, title = field.label, **kwargs)
        return form

    def create_view_form(self, dict = {}):
        form = Form(enctype = "multipart/form-data")
        session = get_session()
        for field in self.fields:
            kwargs = {}
            if field.type == 'title':
                form.widgets.append(HtmlWidget(htmltext('<h3>%s</h3>' % field.label)))
                continue
            if field.type == 'subtitle':
                form.widgets.append(HtmlWidget(htmltext('<h4>%s</h4>' % field.label)))
                continue
            if field.type == 'comment':
                form.widgets.append(HtmlWidget(htmltext('<p>%s</p>' % field.label)))
                continue

            widget_class = widget_classes[field.type]
            value = dict.get(field.label, '')
            if field.type == 'item':
                widget_class = StringWidget

            if field.type == 'file':
                kwargs['preview'] = True
            if field.type == 'bool':
                kwargs['disabled'] = 'disabled'

            if widget_extra_attributes.has_key(field.type):
                for k in widget_extra_attributes[field.type]:
                    v = field.extra.get(k)
                    if v:
                        kwargs[k] = v

            if field.type == 'file':
                widget_class = FakeFileWidget

            form.add(widget_class, 'f%d' % field.id, title = field.label,
                    value = value, readonly = 'readonly', **kwargs)
        return form

    def get_data(self, form):
        d = {}
        for field in self.fields:
            widget = form.get_widget('f%s' % field.id)
            if widget:
                d[field.id] = widget.parse()
        return d

    def notify_new(self, formdata):
        if not self.receiver.emails:
            return
        url = '%s/%s/status' % (get_request().get_url(1), formdata.id)
        mail_body = _("""Hi,

A new form has been submitted on the website, you can consult it with this
link: %(url)s
""") % {'url': url}
        if self.detailed_emails:
            details = []
            if formdata.user:
                details.append(_('User name'))
                details.append('  %s' % formdata.user.name)
            fields = self.fields
            for field in self.fields:
                data = formdata.data
                if not data.has_key(field.id):
                    continue
                details.append(field.label)
                if field.type == 'bool':
                    if data[field.id]:
                        details.append('  %s' % _('Yes'))
                    else:
                        details.append('  %s' % _('No'))
                elif field.type == 'file':
                    file_url = url.replace('status', 'download?f=%d' % field.id)
                    details.append('  %s' % file_url)
                elif field.type == 'text':
                    # XXX: howto support preformatted text in a dl in docutils ?
                    details.append(data[field.id].replace('\n', '\n  '))
                else:
                    details.append('  %s' % data[field.id])
            mail_body += '\n\n' + '\n'.join(details)

        emails.email(_("New form (%s)") % self.name, mail_body,
                self.receiver.emails[0].email,
                bcc = [x.email for x in self.receiver.emails[1:]])

    def notify_change_user(self, formdata, old_status):
        if not formdata.user:
            return
        if not formdata.user.email:
            return

        if self.receiver.emails:
            from_email = self.receiver.emails[0].email
        else:
            from_email = None
        
        url = '%s/' % get_request().get_url(1)
        if old_status != formdata.status:
            mail_body = _("""Hi,

Status of the form you submitted just changed (from "%(before)s" to "%(after)s"),
you can consult it with this link:

%(url)s
""") % {'url': url,
            'before': _(status_labels[old_status]),
            'after': _(status_labels[formdata.status])}
        else:
            mail_body = _("""Hi,

New information has been submitted on the form you submitted, you can
consult it with this link:

%(url)s
""") % {'url': url}

        mail_body += self.get_detailed_evolution(formdata)

        emails.email(_('Form status change'), mail_body, formdata.user.email,
                replyto = from_email)

    def get_detailed_evolution(self, formdata):
        if not self.detailed_emails or not formdata.evolution:
            return ''

        details = []
        evo = formdata.evolution[-1]
        if evo.who:
            details.append(_('User name'))
            details.append('  %s' % evo.who.name)
        if evo.status:
            details.append(_('Status'))
            details.append('  %s' % _(status_labels[evo.status]))
        if evo.comment:
            details.append('\n%s\n' % evo.comment)
        return '\n\n----\n\n' + '\n'.join(details)


    def notify_change_receiver(self, formdata):
        if not self.receiver.emails:
            return

        url = '%s/status' % get_request().get_url(1)
        mail_body = _("""Hi,

A form just changed, you can consult it with this link:

%(url)s
""") % {'url': url}
        mail_body += self.get_detailed_evolution(formdata)
            
        emails.email(_('Form status change (%s)') % self.name, mail_body,
                [x.email for x in self.receiver.emails])

    def migrateOldData(cls):
        import storage
        from roles import Role, get_logged_users_role
        from categories import Category

        init_method = cls.__init__
        cls.__init__ = lambda x: x
        formdefs = storage.get_storage().values('formdefs')
        items = [x.__dict__ for x in formdefs]
        cls.__init__ = init_method

        items.sort(lambda x,y: cmp(x['id'], y['id']))
        logged_users = get_logged_users_role()
        for item in items:
            formdef = FormDef(name = item['name'])
            formdef.url_name = item['id']
            try:
                formdef.receiver = Role.get(item['receiver'])
            except SQLObjectNotFound:
                formdef.receiver = None
            if item.get('category'):
                formdef.category = Category.get(item['category'])
            for role in item.get('roles', []):
                if role == 'logged-users':
                    formdef.addRole(logged_users)
                    continue
                formdef.addRole(Role.get(role))
            formdef.discussion = item.get('discussion', False)
            formdef.confirmation = item.get('confirmation', True)
            formdef.public = item.get('public', False)
            formdef.detailed_emails = item.get('detailed_emails', False)
            for i, field in enumerate(item['fields']):
                fieldObject = FormField(label = field['name'], type = field['type'],
                        formdef = formdef)
                fieldObject.required = field.get('required', False)
                fieldObject.position = i
                d = field.copy()
                del d['name']
                del d['type']
                if d.has_key('required'):
                    del d['required']
                if d:
                    fieldObject.extra = d
            
            # convert submitted forms
            name = 'form-%s' % item['id']
            newname = 'formtmp-%s' % formdef.id
            storage.get_storage().rename_table(name, newname)

    migrateOldData = classmethod(migrateOldData)

